@@ -1,6 +1,6 @@
 #| repl.jl -- rep input loop
 
-   $Id: repl.jl,v 1.45 2001/09/03 03:34:31 jsh Exp $
+   $Id: repl.jl,v 1.46 2001/10/20 06:02:37 jsh Exp $
 
    Copyright (C) 2000 John Harper <john@dcs.warwick.ac.uk>
 
@@ -301,7 +301,7 @@
   (define-repl-command 'collect garbage-collect)
 
   (define-repl-command
-   'dis
+   'disassemble
    (lambda (arg)
      (require 'rep.vm.disassembler)
      (disassemble (repl-eval arg)))
