@@ -1,6 +1,6 @@
 /* lisp.c -- Core of the Lisp, reading and evaluating...
    Copyright (C) 1993, 1994 John Harper <john@dcs.warwick.ac.uk>
-   $Id: lisp.c,v 1.138 2001/09/01 06:21:55 jsh Exp $
+   $Id: lisp.c,v 1.139 2001/10/20 06:00:39 jsh Exp $
 
    This file is part of Jade.
 
@@ -781,9 +781,10 @@ read_str(repv strm, int *c_p)
 static repv
 skip_chars (repv stream, const char *str, repv ret, int *ptr)
 {
+    int c;
     while (*str != 0)
     {
-	int c = rep_stream_getc (stream);
+	c = rep_stream_getc (stream);
 	if (c != *str++)
 	{
 	    char buf[256];
@@ -795,8 +796,22 @@ skip_chars (repv stream, const char *str, repv ret, int *ptr)
 	    return signal_reader_error (Qinvalid_read_syntax, stream, buf);
 	}
     }
-    *ptr = rep_stream_getc (stream);
-    return ret;
+
+    c = rep_stream_getc (stream);
+    switch (c)
+    {
+    case EOF:
+    case ' ': case '\t': case '\n': case '\f': case '\r':
+    case '(': case ')': case '[': case ']':
+    case '\'': case '"': case ';': case ',':
+    case '`':
+	*ptr = c;
+	return ret;
+
+    default:
+	return signal_reader_error (Qinvalid_read_syntax, stream,
+				    "expected end of token");
+    }
 }
 
 /* Using the above readlisp*() functions this classifies each type
