@@ -1,6 +1,6 @@
 /* lispcmds.c -- Lots of standard Lisp functions
    Copyright (C) 1993, 1994 John Harper <john@dcs.warwick.ac.uk>
-   $Id: lispcmds.c,v 1.107 2001/04/30 20:34:52 jsh Exp $
+   $Id: lispcmds.c,v 1.109 2001/08/08 06:15:32 jsh Exp $
 
    This file is part of Jade.
 
@@ -940,10 +940,10 @@ can be a vector or a string. INDEX starts at zero.
 
 DEFUN("make-string", Fmake_string, Smake_string, (repv len, repv init), rep_Subr2) /*
 ::doc:rep.data#make-string::
-make-string LENGTH [INITIAL-repv]
+make-string LENGTH [INITIAL-VALUE]
 
 Returns a new string of length LENGTH, each character is initialised to
-INITIAL-repv, or to space if INITIAL-repv is not given.
+INITIAL-repv, or to space if INITIAL-VALUE is not given.
 ::end:: */
 {
     repv res;
@@ -971,6 +971,7 @@ All indices start at zero.
     int slen;
     rep_DECLARE1(string, rep_STRINGP);
     rep_DECLARE2(start, rep_INTP);
+    rep_DECLARE3_OPT(end, rep_INTP);
     slen = rep_STRING_LEN(string);
     if(rep_INT(start) > slen || rep_INT(start) < 0)
 	return(rep_signal_arg_error(start, 2));
@@ -1277,7 +1278,6 @@ within STRUCTURE. The value of the last form evaluated is returned.
     /* Create the lexical environment for the file. */
     lc.fun = Qnil;
     lc.args = Qnil;
-    lc.args_evalled_p = Qnil;
     rep_PUSH_CALL (lc);
     rep_env = Qnil;
     rep_structure = structure;
@@ -1328,7 +1328,6 @@ DEFUN ("load-dl-file", Fload_dl_file, Sload_dl_file,
     /* Create the lexical environment for the file. */
     lc.fun = Qnil;
     lc.args = Qnil;
-    lc.args_evalled_p = Qnil;
     rep_PUSH_CALL (lc);
     rep_env = Qnil;
     rep_structure = structure;
