@@ -1,5 +1,5 @@
 /* repgdbm.c -- rep wrapper to libgdbm
-   $Id: repgdbm.c,v 1.8 2001/06/18 17:58:09 jsh Exp $ */
+   $Id: repgdbm.c,v 1.9 2001/10/20 06:00:40 jsh Exp $ */
 
 #define _GNU_SOURCE
 
@@ -152,7 +152,7 @@ gdbm-delete DBM KEY
 
 DEFUN("gdbm-walk", Fgdbm_walk, Sgdbm_walk, (repv fun, repv dbm), rep_Subr2) /*
 ::doc:rep.io.db.gdbm#gdbm-walk::
-gdbm-walk DBM FUN
+gdbm-walk FUN DBM
 ::end:: */
 {
     rep_GC_root gc_dbm, gc_fun;
