@@ -1,6 +1,6 @@
 /* unix_main.c -- Miscellaneous functions for Unix
    Copyright (C) 1993, 1994 John Harper <john@dcs.warwick.ac.uk>
-   $Id: unix_main.c,v 1.64 2001/04/23 06:43:01 jsh Exp $
+   $Id: unix_main.c,v 1.65 2001/08/08 06:15:32 jsh Exp $
 
    This file is part of Jade.
 
@@ -795,9 +795,9 @@ fatal_signal_handler(int sig)
 
     /* Try and output the Lisp call stack; this may or may not
        provoke another error, but who cares.. */
-    fprintf(stderr, "\nLisp backtrace:");
+    fprintf(stderr, "\nLisp backtrace:\n");
     Fbacktrace(Fstderr_file());
-    fputs("\n\n", stderr);
+    fputs("\n", stderr);
 
     /* Now reraise the signal, since it's currently blocked
        the default action will occur, i.e. termination */
@@ -843,9 +843,9 @@ usr_signal_handler (int sig)
     switch (sig)
     {
     case SIGUSR1:
-	fprintf(stderr, "\n\nLisp backtrace:");
+	fprintf(stderr, "\n\nLisp backtrace:\n");
 	Fbacktrace(Fstderr_file());
-	fputs("\n\n\n", stderr);
+	fputs("\n\n", stderr);
 	break;
 
     case SIGUSR2:
